==> santiago-0.14.0.crate: santiago-0.14.0/tests/run.rs <==
pub mod ambiguous_integer_addition;
pub mod integer_addition;

#[test]
fn ambiguous_integer_addition() {
    run(
        "ambiguous_integer_addition",
        &ambiguous_integer_addition::lexer::lexer(),
        &ambiguous_integer_addition::grammar::grammar(),
    );
}

#[test]
fn integer_addition() {
    run(
        "integer_addition",
        &integer_addition::lexer::lexer(),
        &integer_addition::grammar::grammar(),
    );
}

#[test]
fn language_nix() {
    let lexer = santiago::languages::nix::lexer_rules();
    let grammar = santiago::languages::nix::grammar_rules(&lexer);

    run("language_nix", &lexer, &grammar);
}

fn run(
    name: &str,
    lexer_rules: &[santiago::lexer::LexerRule],
    grammar: &santiago::grammar::Grammar,
) {
    use std::io::Write;
    let should_update = std::env::var("UPDATE").is_ok();

    let cases_dir = format!("tests/{name}/cases");

    let cases: Vec<String> = std::fs::read_dir(&cases_dir)
        .unwrap()
        .map(|entry| entry.unwrap().file_name().into_string().unwrap())
        .collect();

    for case in cases {
        let path_input = format!("{cases_dir}/{case}/input");
        let path_lexemes = format!("{cases_dir}/{case}/lexemes");
        let path_earley = format!("{cases_dir}/{case}/earley");
        let path_forest = format!("{cases_dir}/{case}/forest");

        let input = std::fs::read_to_string(&path_input).unwrap();

        let lexemes = santiago::lexer::lex(lexer_rules, &input);
        let lexemes_str: String = lexemes
            .iter()
            .map(santiago::lexer::Lexeme::to_string)
            .collect::<Vec<String>>()
            .join("\n");

        let earley = santiago::parser::earley(grammar, &lexemes);
        let earley_str: String = earley
            .iter()
            .map(|column| format!("{column}"))
            .collect::<Vec<String>>()
            .join("\n");

        let forest = santiago::parser::parse(grammar, &lexemes).unwrap();
        let forest_str: String = forest
            .iter()
            .map(|ast| format!("---\n{ast}"))
            .collect::<String>()
            .lines()
            .collect::<Vec<&str>>()
            .join("\n");

        if should_update {
            std::fs::File::create(&path_lexemes)
                .unwrap()
                .write_all(lexemes_str.as_bytes())
                .unwrap();
            std::fs::File::create(&path_earley)
                .unwrap()
                .write_all(earley_str.as_bytes())
                .unwrap();
            std::fs::File::create(&path_forest)
                .unwrap()
                .write_all(forest_str.as_bytes())
                .unwrap();
        }

        assert_eq!(
            lexemes_str,
            std::fs::read_to_string(&path_lexemes).unwrap()
        );
        assert_eq!(earley_str, std::fs::read_to_string(&path_earley).unwrap());
        assert_eq!(forest_str, std::fs::read_to_string(&path_forest).unwrap());
    }
}

==> santiago-0.15.0.crate: santiago-0.15.0/tests/run.rs <==
// SPDX-FileCopyrightText: 2022 Kevin Amado <kamadorueda@gmail.com>
//
// SPDX-License-Identifier: GPL-3.0-only

pub mod ambiguous_integer_addition;
pub mod integer_addition;
pub mod javascript_string_interpolation;
pub mod smallest;

#[test]
fn ambiguous_integer_addition() {
    run(
        "ambiguous_integer_addition",
        &ambiguous_integer_addition::lexer::lexer(),
        &ambiguous_integer_addition::grammar::grammar(),
    );
}

#[test]
fn javascript_string_interpolation() {
    run(
        "javascript_string_interpolation",
        &javascript_string_interpolation::lexer::lexer(),
        &javascript_string_interpolation::grammar::grammar(),
    );
}

#[test]
fn integer_addition() {
    run(
        "integer_addition",
        &integer_addition::lexer::lexer(),
        &integer_addition::grammar::grammar(),
    );
}

#[test]
fn language_nix() {
    let lexer = santiago::languages::nix::lexer_rules();
    let grammar = santiago::languages::nix::grammar_rules(&lexer);

    run("language_nix", &lexer, &grammar);
}

#[test]
fn smallest() {
    run("smallest", &smallest::lexer::lexer(), &smallest::grammar::grammar());
}

fn run(
    name: &str,
    lexer_rules: &[santiago::lexer::LexerRule],
    grammar: &santiago::grammar::Grammar,
) {
    use std::io::Write;
    let should_update = std::env::var("UPDATE").is_ok();

    let cases_dir = format!("tests/{name}/cases");

    let cases: Vec<String> = std::fs::read_dir(&cases_dir)
        .unwrap()
        .map(|entry| entry.unwrap().file_name().into_string().unwrap())
        .collect();

    for case in cases {
        println!("{cases_dir}/{case}");
        let path_input = format!("{cases_dir}/{case}/input");
        let path_lexemes = format!("{cases_dir}/{case}/lexemes");
        let path_earley = format!("{cases_dir}/{case}/earley");
        let path_forest = format!("{cases_dir}/{case}/forest");

        let input = std::fs::read_to_string(&path_input)
            .unwrap()
            .trim_end_matches('\n')
            .to_string();

        let lexemes = santiago::lexer::lex(lexer_rules, &input);
        let lexemes_str: String = lexemes
            .iter()
            .map(santiago::lexer::Lexeme::to_string)
            .collect::<Vec<String>>()
            .join("\n");

        if should_update {
            std::fs::File::create(&path_lexemes)
                .unwrap()
                .write_all(lexemes_str.as_bytes())
                .unwrap();
        }

        let earley = santiago::parser::earley(grammar, &lexemes);
        let earley_str: String = earley
            .iter()
            .map(|column| format!("{column}"))
            .collect::<Vec<String>>()
            .join("\n");

        if should_update {
            std::fs::File::create(&path_earley)
                .unwrap()
                .write_all(earley_str.as_bytes())
                .unwrap();
        }

        let forest = santiago::parser::parse(grammar, &lexemes).unwrap();
        let forest_str: String = forest
            .iter()
            .map(|ast| format!("---\n{ast}"))
            .collect::<String>()
            .lines()
            .collect::<Vec<&str>>()
            .join("\n");

        if should_update {
            std::fs::File::create(&path_forest)
                .unwrap()
                .write_all(forest_str.as_bytes())
                .unwrap();
        }

        assert_eq!(
            lexemes_str,
            std::fs::read_to_string(&path_lexemes).unwrap()
        );
        assert_eq!(earley_str, std::fs::read_to_string(&path_earley).unwrap());
        assert_eq!(forest_str, std::fs::read_to_string(&path_forest).unwrap());
    }
}
